import { world, system } from "@minecraft/server";


// Ensure a new objective.
const swScoreboardName = "Server Waypoints"
const waypointStringPrefix = "ᾧ";
const waypointStringDelimiter = "῝";
const swScoreboardId = waypointStringPrefix + swScoreboardName;

class _WaypointManager {
  constructor() {
    this.swScoreboard = world.scoreboard.getObjective(swScoreboardId);

    if (!this.swScoreboard) {
      this.swScoreboard = world.scoreboard.addObjective(swScoreboardId, swScoreboardName);
    }

    this.pwScoreboards = {};
  }

  /*clearWaypoints() {
    this.swScoreboard.getParticipants().forEach(participant => {
      this.swScoreboard.removeParticipant(participant);
    });
  }*/

  getServerWaypoints() {
    return this.swScoreboard.getParticipants().map((scoreboardIdentity) => this.stringToWaypoint(scoreboardIdentity.displayName));
  }

  getWaypoints(player) {
    return this._getPlayerScoreboard(player).getParticipants().map((scoreboardIdentity) => this.stringToWaypoint(scoreboardIdentity.displayName));
  }

  _getPlayerScoreboard(player) {
    let sb = this.pwScoreboards[player.id]
    if (!sb) {
      sb = world.scoreboard.getObjective(waypointStringPrefix + player.id);
      if (!sb) {
        sb = world.scoreboard.addObjective(waypointStringPrefix + player.id, player.nameTag + " Waypoints");
      }
      this.pwScoreboards[player.id] = sb;
    } 
    
    return sb;
  }

  addServerWaypoint(waypoint, score = 0) {
    this.swScoreboard.setScore(this.waypointToString(waypoint), score);
  }

  addWaypoint(player, waypoint, score = 0) {
    this._getPlayerScoreboard(player).setScore(this.waypointToString(waypoint), score);
  }

  removeServerWaypoint(waypoint) {
    this.swScoreboard.removeParticipant(this.waypointToString(waypoint));
  }

  removeWaypoint(player, waypoint) {
    this._getPlayerScoreboard(player).removeParticipant(this.waypointToString(waypoint));
  }

  removeServerWaypointByName(waypointStr) {
    this.swScoreboard.removeParticipant(waypointStr);
  }

  removeWaypointByName(player, waypointStr) {
    this._getPlayerScoreboard(player).removeParticipant(waypointStr);
  }

  createWaypoint(x, y, z, dimension, xDir = undefined, name = "Waypoint", isPublic = false) {
    if (x == undefined || y == undefined || z == undefined || dimension == undefined)
        return;    

    return {
        x: x,
        y: y,
        z: z,
        xDir: xDir,
        dimension: dimension,
        name: name,
        isPublic: isPublic
    }
  }

  waypointToString(waypoint) {
    let waypointStr = [waypointStringPrefix, waypoint.x, waypoint.y, waypoint.z, waypoint.dimension, waypoint.xDir, waypoint.name, (waypoint.isPublic ? "true" : "false")].join(waypointStringDelimiter);

    return waypointStr;
  }

  stringToWaypoint(waypointStr) {
    let waypointData = waypointStr.split(waypointStringDelimiter);

    // Cleanup the look directions if undefined
    waypointData[5] = !waypointData[5] ? undefined : waypointData[5];

    return this.createWaypoint(waypointData[1], waypointData[2], waypointData[3], waypointData[4], waypointData[5], waypointData[6], waypointData[7] == "true" ? true : false)
  }

}

const WaypointManager = new _WaypointManager();
export default WaypointManager;